"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require('fs');
var path = require('path');
var ramda_1 = __importDefault(require("ramda"));
var acorn_1 = require("./acorn");
var node_ops_1 = require("./node-ops");
var extractProp = function (propNode) {
    return {
        default: undefined,
        name: ramda_1.default.path(['key', 'name'], propNode),
        type: ramda_1.default.path(['value', 'object', 'property', 'name'], propNode),
    };
};
var extractComponent = function (node) {
    var componentClass = node_ops_1.searchBySuperClass(node, 'Component');
    if (componentClass !== undefined) {
        var className = componentClass.id.name;
        var props = node_ops_1.searchForPropTypes(node, className).map(extractProp);
        return {
            componentName: className,
            definition: node,
            props: props,
        };
    }
};
var processFile = function (filePath) {
    return new Promise(function (resolve, reject) {
        fs.readFile(filePath, function (err, content) {
            if (err) {
                reject(err);
            }
            else {
                var ast = acorn_1.parse(content);
                resolve({
                    ast: ast,
                    component: extractComponent(ast),
                    content: content,
                    fileName: path.basename(filePath),
                    filePath: filePath,
                });
            }
        });
    });
};
var SourceFileCache = /** @class */ (function () {
    function SourceFileCache() {
        this.data = new Map();
    }
    SourceFileCache.prototype.getFile = function (filePath) {
        if (this.data.has(filePath)) {
            return this.data.get(filePath);
        }
        else {
            var parsePromise = processFile(filePath);
            this.data.set(filePath, parsePromise);
            return parsePromise;
        }
    };
    return SourceFileCache;
}());
exports.SourceFileCache = SourceFileCache;
//# sourceMappingURL=tree-view.js.map